@@ -1,11 +1,10 @@
+from typing import Optional
+
 import can.interfaces.ixxat.canlib_vcinpl as vcinpl
 import can.interfaces.ixxat.canlib_vcinpl2 as vcinpl2
-
 from can import BusABC, Message
 from can.bus import BusState
 
-from typing import Optional
-
 
 class IXXATBus(BusABC):
     """The CAN Bus implemented for the IXXAT interface.
@@ -146,8 +145,9 @@ class IXXATBus(BusABC):
     def _send_periodic_internal(self, msgs, period, duration=None):
         return self.bus._send_periodic_internal(msgs, period, duration)
 
-    def shutdown(self):
-        return self.bus.shutdown()
+    def shutdown(self) -> None:
+        super().shutdown()
+        self.bus.shutdown()
 
     @property
     def state(self) -> BusState:
@@ -155,6 +155,3 @@ class IXXATBus(BusABC):
         Return the current state of the hardware
         """
         return self.bus.state
-
-
-# ~class IXXATBus(BusABC): ---------------------------------------------------
