@@ -3,9 +3,8 @@
 """
 """
 
-from time import sleep
-import unittest
 import logging
+import unittest
 
 import can
 
@@ -14,8 +13,8 @@ logging.getLogger(__file__).setLevel(logging.DEBUG)
 
 class ZeroDLCTest(unittest.TestCase):
     def test_recv_non_zero_dlc(self):
-        bus_send = can.interface.Bus(bustype="virtual")
-        bus_recv = can.interface.Bus(bustype="virtual")
+        bus_send = can.interface.Bus(interface="virtual")
+        bus_recv = can.interface.Bus(interface="virtual")
         data = [0, 1, 2, 3, 4, 5, 6, 7]
         msg_send = can.Message(is_extended_id=False, arbitration_id=0x100, data=data)
 
@@ -26,7 +25,7 @@ class ZeroDLCTest(unittest.TestCase):
         self.assertTrue(msg_recv)
 
     def test_recv_none(self):
-        bus_recv = can.interface.Bus(bustype="virtual")
+        bus_recv = can.interface.Bus(interface="virtual")
 
         msg_recv = bus_recv.recv(timeout=0)
 
@@ -34,8 +33,8 @@ class ZeroDLCTest(unittest.TestCase):
         self.assertFalse(msg_recv)
 
     def test_recv_zero_dlc(self):
-        bus_send = can.interface.Bus(bustype="virtual")
-        bus_recv = can.interface.Bus(bustype="virtual")
+        bus_send = can.interface.Bus(interface="virtual")
+        bus_recv = can.interface.Bus(interface="virtual")
         msg_send = can.Message(is_extended_id=False, arbitration_id=0x100, data=[])
 
         bus_send.send(msg_send)
